//+------------------------------------------------------------------+
//|                                                SummaryReport.mq4 |
//|                      Copyright  2006, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#define OP_BALANCE 6
#define OP_CREDIT  7
#define MaxCountParameters 64
#define MaxCountSystem 128

//#include OptomizationReport3

extern bool OR.bProfit        = true;
extern string OR.delimiter    =";";
extern bool OR.OrderCommission=1;
extern bool OR.OrderSwap      =1;


string OR.NameSystem[MaxCountSystem];          //      
string OR.VarSystem[MaxCountSystem];           //     
int    OR.CountSystem=0;                       //     (  MaxCountParameters)

string OR.NameParameter[MaxCountParameters];   //      
double OR.VarParameter[MaxCountParameters];    //     
int    OR.CountValue=0;                        //     (  MaxCountParameters)



double OR.InitialDeposit;     // Initial deposit
double OR.SummaryProfit;      // Total net profit
double OR.GrossProfit;        // Gross profit
double OR.GrossLoss;          // Gross loss
double OR.MaxProfit;          // Largest profit trade
double OR.MinProfit;          // Largest loss trade
double OR.ConProfit1;         // Maximum consecutive wins (profit in money)
double OR.ConProfit2;         // Maximal consecutive profit
double OR.ConLoss1;           // Maximum consecutive losses (loss in money)
double OR.ConLoss2;           // Maximal consecutive loss
double OR.MaxLoss;            
double OR.MaxDrawdown;        // Maximal drawdown
double OR.MaxDrawdownPercent; // MaxDrawdownPercent
double OR.RelDrawdownPercent; // RelDrawdownPercent
double OR.RelDrawdown;        // Relative drawdown
double OR.ExpectedPayoff;     // Expected payoff
double OR.ProfitFactor;       // Profit factor
double OR.AbsoluteDrawdown;   // Absolute drawdown
int    OR.SummaryTrades;      // Trades total
int    OR.ProfitTrades;       // Profit trades
int    OR.LossTrades;         // Loss trades 
int    OR.ShortTrades;        // Short positions
int    OR.LongTrades;         // Long positions
int    OR.WinShortTrades;
int    OR.WinLongTrades;
int    OR.ConProfitTrades1;   // Maximum consecutive wins
int    OR.ConProfitTrades2;   // Maximal consecutive profit (count of wins)
int    OR.ConLossTrades1;     // Maximum consecutive losses
int    OR.ConLossTrades2;     // Maximal consecutive loss (count of losses)
int    OR.AvgConWinners;      // Average consecutive wins
int    OR.AvgConLosers;       // Average consecutive losses

double OR.Balans_Money[], OR.MNK_Money[3];


void OR.main(){
   OR.InitialDeposit=CalculateInitialDeposit();
   if (OR.bProfit)
      if (AccountBalance()<OR.InitialDeposit)
         return;
   CalculateSummary(OR.InitialDeposit);
   OR.SaveSystem();
   OR.WriteReport();
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CalculateSummary(double initial_deposit)   {
   int    sequence=0, profitseqs=0, lossseqs=0;
   int count_money=0, count_point=0;
   double sequential=0.0, prevprofit=EMPTY_VALUE, drawdownpercent, drawdown;

   double maxpeak=initial_deposit, minpeak=initial_deposit, balance=initial_deposit;
   double _balance=0;
   OR.Balans_Money[count_money]=0;
   count_money++;
   int    trades_total=HistoryTotal();
//---- initialize summaries
   InitializeSummaries(initial_deposit);
   ArrayResize(OR.Balans_Money,trades_total);

   for(int i=0; i<trades_total; i++) {
      if(!OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)) continue;
      int type=OrderType();
      
      //---- initial balance not considered
      if(i==0 && type==OP_BALANCE) continue;
      
      //---- calculate profit
      double profit=OrderProfit()+OrderCommission()*OR.OrderCommission+OrderSwap()*OR.OrderSwap;
      
      balance+=profit;
      _balance+=profit;
      if (OrderType()<2) {
         OR.Balans_Money[count_money]=_balance;
         count_money++;
      }
      //---- drawdown check
      if(maxpeak<balance) {
         drawdown=maxpeak-minpeak;
         if(maxpeak!=0.0) {
            drawdownpercent=drawdown/maxpeak*100.0;
            if(OR.RelDrawdownPercent<drawdownpercent) {
               OR.RelDrawdownPercent=drawdownpercent;
               OR.RelDrawdown=drawdown;
            }
         }
         if(OR.MaxDrawdown<drawdown) {
            OR.MaxDrawdown=drawdown;
            if(maxpeak!=0.0) OR.MaxDrawdownPercent=OR.MaxDrawdown/maxpeak*100.0; else OR.MaxDrawdownPercent=100.0;
         }
         maxpeak=balance;
         minpeak=balance;
      }
      if(minpeak>balance) minpeak=balance;
      if(OR.MaxLoss>balance) OR.MaxLoss=balance;
      //---- market orders only
      if(type!=OP_BUY && type!=OP_SELL) continue;
      //---- calculate profit in points
      // profit=(OrderClosePrice()-OrderOpenPrice())/MarketInfo(OrderSymbol(),MODE_POINT);
      OR.SummaryProfit+=profit;
      OR.SummaryTrades++;
      if(type==OP_BUY) OR.LongTrades++; else OR.ShortTrades++;
      
      if(profit<0) { //---- loss trades
         OR.LossTrades++;
         OR.GrossLoss+=profit;
         if(OR.MinProfit>profit) OR.MinProfit=profit;
         //---- fortune changed
         if(prevprofit!=EMPTY_VALUE && prevprofit>=0) {
            if(OR.ConProfitTrades1<sequence || (OR.ConProfitTrades1==sequence && OR.ConProfit2<sequential)) {
               OR.ConProfitTrades1=sequence;
               OR.ConProfit1=sequential;
            }
            if(OR.ConProfit2<sequential || (OR.ConProfit2==sequential && OR.ConProfitTrades1<sequence)) {
               OR.ConProfit2=sequential;
               OR.ConProfitTrades2=sequence;
            }
            profitseqs++;
            OR.AvgConWinners+=sequence;
            sequence=0;
            sequential=0.0;
         }
      } else { //---- profit trades (profit>=0)
         OR.ProfitTrades++;
         if(type==OP_BUY)  OR.WinLongTrades++;
         if(type==OP_SELL) OR.WinShortTrades++;
         OR.GrossProfit+=profit;
         if(OR.MaxProfit<profit) OR.MaxProfit=profit;
         //---- fortune changed
         if(prevprofit!=EMPTY_VALUE && prevprofit<0) {
            if(OR.ConLossTrades1<sequence || (OR.ConLossTrades1==sequence && OR.ConLoss2>sequential)) {
               OR.ConLossTrades1=sequence;
               OR.ConLoss1=sequential;
            }
            if(OR.ConLoss2>sequential || (OR.ConLoss2==sequential && OR.ConLossTrades1<sequence)) {
               OR.ConLoss2=sequential;
               OR.ConLossTrades2=sequence;
            }
            lossseqs++;
            OR.AvgConLosers+=sequence;
            sequence=0;
            sequential=0.0;
         }
      }
      sequence++;
      sequential+=profit;
      //----
      prevprofit=profit;


   }
//---- final drawdown check
   drawdown=maxpeak-minpeak;
   if(maxpeak!=0.0) {
      drawdownpercent=drawdown/maxpeak*100.0;
      if(OR.RelDrawdownPercent<drawdownpercent) {
         OR.RelDrawdownPercent=drawdownpercent;
         OR.RelDrawdown=drawdown;
      }
   }
   if(OR.MaxDrawdown<drawdown) {
      OR.MaxDrawdown=drawdown;
      if(maxpeak!=0) OR.MaxDrawdownPercent=OR.MaxDrawdown/maxpeak*100.0;
      else OR.MaxDrawdownPercent=100.0;
   }
   //---- consider last trade
   if(prevprofit!=EMPTY_VALUE) {
      profit=prevprofit;
      if(profit<0) {
         if(OR.ConLossTrades1<sequence || (OR.ConLossTrades1==sequence && OR.ConLoss2>sequential)) {
            OR.ConLossTrades1=sequence;
            OR.ConLoss1=sequential;
         }
         if(OR.ConLoss2>sequential || (OR.ConLoss2==sequential && OR.ConLossTrades1<sequence)) {
            OR.ConLoss2=sequential;
            OR.ConLossTrades2=sequence;
         }
         lossseqs++;
         OR.AvgConLosers+=sequence;
      } else {
         if(OR.ConProfitTrades1<sequence || (OR.ConProfitTrades1==sequence && OR.ConProfit2<sequential)) {
            OR.ConProfitTrades1=sequence;
            OR.ConProfit1=sequential;
         }
         if(OR.ConProfit2<sequential || (OR.ConProfit2==sequential && OR.ConProfitTrades1<sequence)) {
            OR.ConProfit2=sequential;
            OR.ConProfitTrades2=sequence;
         }
         profitseqs++;
         OR.AvgConWinners+=sequence;
      }
   }
   //---- collecting done
   double dnum, profitkoef=0.0, losskoef=0.0, avgprofit=0.0, avgloss=0.0;
   
   //---- average consecutive wins and losses
   dnum=OR.AvgConWinners;
   if(profitseqs>0) OR.AvgConWinners=dnum/profitseqs+0.5;
   dnum=OR.AvgConLosers;
   if(lossseqs>0)   OR.AvgConLosers=dnum/lossseqs+0.5;
   
   //---- absolute values
   if(OR.GrossLoss<0.0) OR.GrossLoss*=-1.0;
   if(OR.MinProfit<0.0) OR.MinProfit*=-1.0;
   if(OR.ConLoss1<0.0)  OR.ConLoss1*=-1.0;
   if(OR.ConLoss2<0.0)  OR.ConLoss2*=-1.0;
   
   //---- profit factor
   if(OR.GrossLoss>0.0) OR.ProfitFactor=OR.GrossProfit/OR.GrossLoss;
   
   //---- expected payoff
   if(OR.ProfitTrades>0) avgprofit=OR.GrossProfit/OR.ProfitTrades;
   if(OR.LossTrades>0)   avgloss  =OR.GrossLoss/OR.LossTrades;
   if(OR.SummaryTrades>0) {
      profitkoef=1.0*OR.ProfitTrades/OR.SummaryTrades;
      losskoef=1.0*OR.LossTrades/OR.SummaryTrades;
      OR.ExpectedPayoff=profitkoef*avgprofit-losskoef*avgloss;
   }
   //---- absolute drawdown
   OR.AbsoluteDrawdown=initial_deposit-OR.MaxLoss;
   
   ArrayResize(OR.Balans_Money,count_money);
   MNK(OR.Balans_Money,OR.MNK_Money);
   SKO(OR.Balans_Money,OR.MNK_Money);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void InitializeSummaries(double initial_deposit) {
   OR.InitialDeposit       = initial_deposit;
   OR.MaxLoss              = initial_deposit;
   OR.SummaryProfit        = 0.0;
   OR.GrossProfit          = 0.0;
   OR.GrossLoss            = 0.0;
   OR.MaxProfit            = 0.0;
   OR.MinProfit            = 0.0;
   OR.ConProfit1           = 0.0;
   OR.ConProfit2           = 0.0;
   OR.ConLoss1             = 0.0;
   OR.ConLoss2             = 0.0;
   OR.MaxDrawdown          = 0.0;
   OR.MaxDrawdownPercent   = 0.0;
   OR.RelDrawdownPercent   = 0.0;
   OR.RelDrawdown          = 0.0;
   OR.ExpectedPayoff       = 0.0;
   OR.ProfitFactor         = 0.0;
   OR.AbsoluteDrawdown     = 0.0;
   OR.SummaryTrades        = 0;
   OR.ProfitTrades         = 0;
   OR.LossTrades           = 0;
   OR.ShortTrades          = 0;
   OR.LongTrades           = 0;
   OR.WinShortTrades       = 0;
   OR.WinLongTrades        = 0;
   OR.ConProfitTrades1     = 0;
   OR.ConProfitTrades2     = 0;
   OR.ConLossTrades1       = 0;
   OR.ConLossTrades2       = 0;
   OR.AvgConWinners        = 0;
   OR.AvgConLosers         = 0;
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CalculateInitialDeposit() {
   double initial_deposit=AccountBalance();
   for(int i=HistoryTotal()-1; i>=0; i--) {
      if(!OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)) continue;
      int type=OrderType();
      if(i==0 && type==OP_BALANCE) break;
      if(type==OP_BUY || type==OP_SELL) {
         
         //---- calculate profit
         double profit=OrderProfit()+OrderCommission()*OR.OrderCommission+OrderSwap()*OR.OrderSwap;
         
         //---- and decrease balance
         initial_deposit-=profit;
      }
      if(type==OP_BALANCE || type==OP_CREDIT)
         initial_deposit-=OrderProfit();
   }
   return(initial_deposit);
   
}
  
// ====================================================
//      
//      
// Name           -  
// CurrentValue   -  
// ====================================================
void OR.Add_Parameter(string Name, double CurrentValue){
   if (OR.CountValue<MaxCountParameters) {
      OR.NameParameter[OR.CountValue]=Name;
      OR.VarParameter[OR.CountValue]=CurrentValue;
      OR.CountValue++;
   }
}   
void OR.Add_System(string Name, string CurrentValue){
   if (OR.CountSystem<MaxCountSystem) {
      OR.NameSystem[OR.CountSystem]=Name;
      OR.VarSystem[OR.CountSystem]=CurrentValue;
      OR.CountSystem++;
   }
}   

//       
// Y(x)=b1*x+b0;
// b1=(Sx*Sy-N*Sxy)/(Sx*Sx-N*Sxx)
// b0=(Sy-b1*Sx)/N;

void MNK(double Input[], double & Output[]){
   int Count=ArraySize(Input);
   double   Sx=0, Sxx=0, Sy=0, Sxy=0, y=0;
  
   ArrayInitialize(Output,0);
   
   for (int x = 1; x<=Count;x++) { 
      Sx+=x; 
      y=Input[x-1]; 
      Sxx+=x*x; 
      Sy+=y; 
      Sxy+=x*y; 
   }
   if ((Sx*Sx-Count*Sxx)!=0) { 
      Output[1]=(Sx*Sy-Count*Sxy)/(Sx*Sx-Count*Sxx); 
      Output[0]=(Sy-Output[1]*Sx)/Count;
   }
}

//  - 
void SKO(double Input[], double & Output[]){
   double   dy,SS=0;
   int Count=ArraySize(Input);
   for (int x=1;x<=Count;x++) {   
      dy=(Input[x-1]-x*Output[1]-Output[0]);  
      SS+=dy*dy; 
   }
   SS/=Count;
   Output[2]=MathSqrt(SS);
}

string sTimeFrame(){
   switch (Period()) {
      case PERIOD_M1 : return("M1");
      case PERIOD_M5 : return("M5");
      case PERIOD_M15: return("M15");
      case PERIOD_M30: return("M30");
      case PERIOD_H1 : return("H1");
      case PERIOD_H4 : return("H4");
      case PERIOD_D1 : return("D1");
      case PERIOD_W1 : return("W1");
      case PERIOD_MN1: return("MN1");
   }
}

void OR.WriteReport(){
   int i;
   string ss1="",ss2="",ss3="";
   string ReportName=WindowExpertName()+"_"+Symbol()+"_"+sTimeFrame()+".csv";
   int handle=FileOpen(ReportName,FILE_READ|FILE_WRITE,OR.delimiter);
   if(handle<1) return;
   if (FileSize(handle)==0) {
      for (i=0               ; i<OR.CountSystem/2-1; i++) ss1=StringConcatenate(ss1, OR.NameSystem[i],    OR.delimiter);  ss1=StringConcatenate(ss1, OR.NameSystem[OR.CountSystem/2-1]);      
      for (i=OR.CountSystem/2; i<OR.CountSystem-1  ; i++) ss2=StringConcatenate(ss2, OR.NameSystem[i],    OR.delimiter);  ss2=StringConcatenate(ss2, OR.NameSystem[OR.CountSystem-1]);
      for (i=0               ; i<OR.CountValue-1   ; i++) ss3=StringConcatenate(ss3, OR.NameParameter[i], OR.delimiter);  ss3=StringConcatenate(ss3, OR.NameParameter[OR.CountValue-1]);
      FileWrite(handle, ss1, ss2,ss3,"");
      ss1="";
      ss2="";
      ss3="";
   }
   FileSeek(handle,0,SEEK_END);


   for (i=0               ; i<OR.CountSystem/2-1; i++)   ss1=StringConcatenate(ss1, OR.VarSystem[i],    OR.delimiter);  ss1=StringConcatenate(ss1, OR.VarSystem[OR.CountSystem/2-1]);      
   for (i=OR.CountSystem/2; i<OR.CountSystem-1  ; i++)   ss2=StringConcatenate(ss2, OR.VarSystem[i],    OR.delimiter);  ss2=StringConcatenate(ss2, OR.VarSystem[OR.CountSystem-1]);      
   for (i=0               ; i<OR.CountValue-1   ; i++)   ss3=StringConcatenate(ss3, OR.VarParameter[i], OR.delimiter);  ss3=StringConcatenate(ss3, OR.VarParameter[OR.CountValue-1]);   

   FileWrite(handle, ss1,ss2,ss3,"");

   FileClose(handle);
   
}

void OR.SaveSystem(){
   OR.Add_System("Initial deposit",                            
                  DoubleToStr( OR.InitialDeposit,                          4));
   OR.Add_System("Total net profit",                           
                  DoubleToStr( OR.SummaryProfit,                           4));
   OR.Add_System("Gross profit",                               
                  DoubleToStr( OR.GrossProfit,                             4));
   OR.Add_System("Gross loss",                                 
                  DoubleToStr( OR.GrossLoss,                               4));
   OR.Add_System("Profit factor",                              
                  DoubleToStr( OR.ProfitFactor,                            4));
   OR.Add_System("Expected payoff",                            
                  DoubleToStr( OR.ExpectedPayoff,                          4));
   OR.Add_System("Absolute drawdown",                          
                  DoubleToStr( OR.AbsoluteDrawdown,                        4));
   OR.Add_System("Maximal drawdown",                           
                  DoubleToStr( OR.MaxDrawdown,                             4));
   OR.Add_System("MaxDrawdownPercent",                         
                  DoubleToStr( OR.MaxDrawdownPercent,                      4));
   OR.Add_System("Relative drawdown",                          
                  DoubleToStr( OR.RelDrawdown,                             4));
   OR.Add_System("RelDrawdownPercent",                         
                  DoubleToStr( OR.RelDrawdownPercent,                      4));
   OR.Add_System("Trades total",                               
                  DoubleToStr( OR.SummaryTrades,                           4));
   OR.Add_System("Short positions",                            
                  DoubleToStr( OR.ShortTrades,                             4));
   OR.Add_System("Short positions(won %)",                     
                  DoubleToStr( 100.0*DIV(OR.WinShortTrades,OR.ShortTrades),4));
   OR.Add_System("Long positions",                             
                  DoubleToStr( OR.LongTrades,                              4));
   OR.Add_System("Long positions(won %)",                      
                  DoubleToStr( 100.0*DIV(OR.WinLongTrades,OR.LongTrades),  4));
   OR.Add_System("Profit trades",                              
                  DoubleToStr( OR.ProfitTrades,                            4));
   OR.Add_System("Profit trades (% of total)",                 
                  DoubleToStr( 100.0*DIV(OR.ProfitTrades,OR.SummaryTrades),4));
   OR.Add_System("Loss trades",                                
                  DoubleToStr( OR.LossTrades,                              4));
   OR.Add_System("Loss trades (% of total)",                   
                  DoubleToStr( 100.0*DIV(OR.LossTrades,OR.SummaryTrades),  4));
   OR.Add_System("Largest profit trade",                       
                  DoubleToStr( OR.MaxProfit,                               4));
   OR.Add_System("Largest loss trade",                         
                  DoubleToStr(-OR.MinProfit,                               4));
   OR.Add_System("Average profit trade",                       
                  DoubleToStr( DIV(OR.GrossProfit,OR.ProfitTrades),        4));
   OR.Add_System("Average loss trade",                         
                  DoubleToStr(-DIV(OR.GrossLoss,OR.LossTrades),            4));
   OR.Add_System("Average consecutive wins",                   
                  DoubleToStr( OR.AvgConWinners,                           4));
   OR.Add_System("Average consecutive losses",                 
                  DoubleToStr( OR.AvgConLosers,                            4));
   OR.Add_System("Maximum consecutive wins",                   
                  DoubleToStr( OR.ConProfitTrades1,                        4));
   OR.Add_System("Maximum consecutive wins (profit in money)", 
                  DoubleToStr( OR.ConProfit1,                              4));
   OR.Add_System("Maximum consecutive losses",                 
                  DoubleToStr( OR.ConLossTrades1,                          4));
   OR.Add_System("Maximum consecutive losses (loss in money)", 
                  DoubleToStr(-OR.ConLoss1,                                4));
   OR.Add_System("Maximal consecutive profit",                 
                  DoubleToStr( OR.ConProfit2,                              4));
   OR.Add_System("Maximal consecutive profit (count of wins)", 
                  DoubleToStr( OR.ConProfitTrades2,                        4));
   OR.Add_System("Maximal consecutive loss",                   
                  DoubleToStr(-OR.ConLoss2,                                4));
   OR.Add_System("Maximal consecutive loss (count of losses)", 
                  DoubleToStr( OR.ConLossTrades2,                          4));
//   OR.Add_System("Alfa", 
//                  DoubleToStr( OR.MNK_Money[0],                          4));
   OR.Add_System("Betta", 
                  DoubleToStr( OR.MNK_Money[1],                          4));
   OR.Add_System("SKO", 
                  DoubleToStr( OR.MNK_Money[2],                          4));
                  OR.Add_System("RECOVERY FACTOR ", 
                  DoubleToStr( OR.SummaryProfit/OR.MaxDrawdown,            4));
   OR.Add_System(" ", 
                  DoubleToStr( 123321,                                      4));
   
    GlobalVariableSet("myValue",OR.SummaryProfit/OR.MaxDrawdown);
//Print(" rf ",rf);

}

double DIV(double A, double B){
   if (MathAbs(B)>0) return(A/B);
   return(0);
}




